package com.starbuds.server.service.net.impl;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.ContentType;
import com.alibaba.fastjson.JSON;
import com.starbuds.server.common.pojo.net.NetResponse;
import com.starbuds.server.common.service.net.RequestService;
import com.starbuds.server.service.net.okhttp.AbstractOkHttpService;
import com.starbuds.server.service.net.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Response;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 请求服务实现
 *
 * @author Lay
 * @date 2020/6/4
 */
@Service(version = "1.0.0")
@Slf4j
public class RequestServiceImpl implements RequestService {

    @Resource
    private AbstractOkHttpService syncOkHttpService;

    @Resource
    private AbstractOkHttpService asyncOkHttpService;

    /**
     * 同步 GET 请求
     *
     * @param url
     * @return
     */
    @Override
    public NetResponse syncGet(String url) {
        return syncGet(url, null);
    }

    /**
     * 同步 GET 请求
     *
     * @param url
     * @param headers
     * @return
     */
    @Override
    public NetResponse syncGet(String url, Map<String, String> headers) {
        return syncGet(url, headers, null, null);
    }

    /**
     * 同步 GET 请求
     *
     * @param url
     * @param headers
     * @param timeout
     * @param timeUnit
     * @return
     */
    @Override
    public NetResponse syncGet(String url, Map<String, String> headers, Long timeout, TimeUnit timeUnit) {
        if (StringUtils.isBlank(url)) {
            log.warn("url 为空");
            return null;
        }
        headers = HttpUtil.setTraceInfo(headers, true);
        String traceId = HttpUtil.getTraceId(headers);
        Response res;
        try {
            res = syncOkHttpService.getResponse(url, headers, timeout, timeUnit);
        } catch (IOException e) {
            log.error(StrUtil.format("同步-GET-traceId-{}失败", traceId), e);
            return null;
        }
        if (res == null) {
            log.error(StrUtil.format("同步-GET-traceId-{}失败 返回结果为 null", traceId));
        }

        return HttpUtil.getNetResponse(res);
    }

    /**
     * 同步 POST 请求
     *
     * @param url
     * @param params
     * @return
     */
    @Override
    public NetResponse syncPost(String url, Map<String, String> params) {
        return syncPost(url, null, params);
    }

    /**
     * 同步 POST 请求
     *
     * @param url
     * @param headers
     * @param params
     * @return
     */
    @Override
    public NetResponse syncPost(String url, Map<String, String> headers, Map<String, String> params) {
        return syncPost(url, headers, params, null, null);
    }

    /**
     * 同步 POST 请求
     *
     * @param url
     * @param headers
     * @param params
     * @param timeout
     * @param timeUnit
     * @return
     */
    @Override
    public NetResponse syncPost(String url, Map<String, String> headers, Map<String, String> params, Long timeout, TimeUnit timeUnit) {
        if (StringUtils.isBlank(url)) {
            log.warn("url 为空");
            return null;
        }

        headers = HttpUtil.setTraceInfo(headers, true);
        String traceId = HttpUtil.getTraceId(headers);
        Response res;
        try {
            res = syncOkHttpService.postResponse(url, params, headers, timeout, timeUnit);
        } catch (IOException e) {
            log.error(StrUtil.format("同步-POST-traceId-{}失败", traceId), e);
            return null;
        }
        if (res == null) {
            log.error(StrUtil.format("同步-POST-traceId-{}失败 返回结果为 null", traceId));
        }

        return HttpUtil.getNetResponse(res);
    }

    @Override
    public NetResponse syncPost(String url, Map<String, String> headers, String body, Long timeout, TimeUnit timeUnit) {
        if (StringUtils.isBlank(url)) {
            log.warn("url 为空");
            return null;
        }

        headers = HttpUtil.setTraceInfo(headers, true);
        String traceId = HttpUtil.getTraceId(headers);
        Response res = null;
        ContentType contentType = ContentType.get(body);
        try {
            switch (contentType) {
                case JSON:
                    res = syncOkHttpService.postJsonResponse(url, headers, body, timeout, timeUnit);
                    break;
                case XML:
                    res = syncOkHttpService.postXmlResponse(url, headers, body, timeout, timeUnit);
                    break;
            }
        } catch (IOException e) {
            log.error(StrUtil.format("同步-POST-traceId-{}失败", traceId), e);
            return null;
        }

        return HttpUtil.getNetResponse(res);
    }

    @Override
    public NetResponse syncPostFile(String url, Map<String, String> headers, byte[] file, String paramName, String fileName, Long timeout, TimeUnit timeUnit) {
        if (StringUtils.isBlank(url)) {
            log.warn("url 为空");
            return null;
        }

        headers = HttpUtil.setTraceInfo(headers, true);
        String traceId = HttpUtil.getTraceId(headers);
        Response res = null;
        try {
            res = syncOkHttpService.postFileResponse(url, headers, file, paramName, fileName, timeout, timeUnit);
        } catch (IOException e) {
            log.error(StrUtil.format("同步-POST-traceId-{}失败", traceId), e);
        }
        return HttpUtil.getNetResponse(res);
    }

    /**
     * 异步 GET 请求
     *
     * @param url
     * @return
     */
    @Override
    public boolean asyncGet(String url) {
        return asyncGet(url, null);
    }

    /**
     * 异步 GET 请求
     *
     * @param url
     * @param headers
     * @return
     */
    @Override
    public boolean asyncGet(String url, Map<String, String> headers) {
        return asyncGet(url, headers, null, null);
    }

    /**
     * 异步 GET 请求
     *
     * @param url
     * @param headers
     * @param timeout
     * @param timeUnit
     * @return
     */
    @Override
    public boolean asyncGet(String url, Map<String, String> headers, Long timeout, TimeUnit timeUnit) {
        if (StringUtils.isBlank(url)) {
            log.warn("url 为空");
            return false;
        }

        headers = HttpUtil.setTraceInfo(headers, false);
        String traceId = HttpUtil.getTraceId(headers);
        try {
            asyncOkHttpService.getResponse(url, headers, timeout, timeUnit);
        } catch (IOException e) {
            log.error(StrUtil.format("异步-GET-traceId-{}失败", traceId), e);
            return false;
        }

        return true;
    }

    /**
     * 异步 POST 请求
     *
     * @param url
     * @param params
     * @return
     */
    @Override
    public boolean asyncPost(String url, Map<String, String> params) {
        return asyncPost(url, null, params);
    }

    /**
     * 异步 POST 请求
     *
     * @param url
     * @param headers
     * @param params
     * @return
     */
    @Override
    public boolean asyncPost(String url, Map<String, String> headers, Map<String, String> params) {
        return asyncPost(url, headers, params, null, null);
    }

    /**
     * 异步 POST 请求
     *
     * @param url
     * @param headers
     * @param params
     * @param timeout
     * @param timeUnit
     * @return
     */
    @Override
    public boolean asyncPost(String url, Map<String, String> headers, Map<String, String> params, Long timeout, TimeUnit timeUnit) {
        if (StringUtils.isBlank(url)) {
            log.warn("url 为空");
            return false;
        }

        headers = HttpUtil.setTraceInfo(headers, false);
        String traceId = HttpUtil.getTraceId(headers);
        try {
            syncOkHttpService.postResponse(url, params, headers, timeout, timeUnit);
        } catch (IOException e) {
            log.error(StrUtil.format("异步-POST-traceId-{}失败", traceId), e);
            return false;
        }

        return true;
    }

    @Override
    public boolean asyncPost(String url, Map<String, String> headers, String body, Long timeout, TimeUnit timeUnit) {
        if (StringUtils.isBlank(url)) {
            log.warn("url 为空");
            return false;
        }

        headers = HttpUtil.setTraceInfo(headers, false);
        String traceId = HttpUtil.getTraceId(headers);
        ContentType contentType = ContentType.get(body);
        try {
            switch (contentType) {
                case JSON:
                    asyncOkHttpService.postJsonResponse(url, headers, body, timeout, timeUnit);
                    break;
                case XML:
                    asyncOkHttpService.postXmlResponse(url, headers, body, timeout, timeUnit);
                    break;
            }
        } catch (IOException e) {
            log.error(StrUtil.format("异步-POST-traceId-{}失败", traceId), e);
            return false;
        }

        return true;
    }

}
