package com.starbuds.server.service.net.okhttp;

import cn.hutool.core.util.StrUtil;
import com.starbuds.server.service.net.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * OkHttp工具类
 *
 * @author aoe
 * @date 2020/6/09
 */
@Slf4j
public abstract class AbstractOkHttpService {
    private static final MediaType JSON_TYPE
            = MediaType.parse("application/json; charset=utf-8");
    private static final MediaType XML_TYPE
            = MediaType.parse("application/xml; charset=utf-8");
    private static final MediaType FILE_TYPE
            = MediaType.parse("application/octet-stream; charset=utf-8");
    private static final MediaType FORM_TYPE
            = MediaType.parse("multipart/form-data; charset=utf-8");

    /**
     * 是否同步方法
     *
     * @return true 同步方法
     */
    protected abstract boolean isSync();

    protected abstract OkHttpClient getClient(Long timeout, TimeUnit timeUnit);

    public Response getResponse(String url, Map<String, String> headers, Long timeout, TimeUnit timeUnit) throws IOException {
        Request.Builder builder = new Request.Builder();
        if (headers != null) {
            for (Map.Entry<String, String> header : headers.entrySet()) {
                builder.addHeader(header.getKey(), header.getValue());
            }
        }
        Request request = builder.url(url).build();
        return excute(request, timeout, timeUnit);
    }

    public Response postResponse(String url, Map<String, String> params, Map<String, String> headers, Long timeout, TimeUnit timeUnit) throws IOException {
        Request.Builder builder = new Request.Builder();
        if (headers != null) {
            for (Map.Entry<String, String> header : headers.entrySet()) {
                builder.addHeader(header.getKey(), header.getValue());
            }
        }

        if (params != null) {
            FormBody.Builder body = new FormBody.Builder();
            for (Map.Entry<String, String> param : params.entrySet()) {
                body.add(param.getKey(), param.getValue());
            }
            builder.post(body.build());
        }

        Request request = builder.url(url).build();
        return excute(request, timeout, timeUnit);
    }

    public Response postJsonResponse(String url, Map<String, String> header, String json, Long timeout, TimeUnit timeUnit) throws IOException {
        return getResponse(url, header, json, timeout, timeUnit, JSON_TYPE);
    }

    public Response postXmlResponse(String url, Map<String, String> header, String xml, Long timeout, TimeUnit timeUnit) throws IOException {
        return getResponse(url, header, xml, timeout, timeUnit, XML_TYPE);
    }

    public Response postFileResponse(String url, Map<String, String> headers, byte[] file, String paramName, String fileName, Long timeout, TimeUnit timeUnit) throws IOException {
        RequestBody fileBody = RequestBody.create(file, FILE_TYPE);
        return postFileResponse(url, headers, fileBody, paramName, fileName, timeout, timeUnit);
    }

    private Response postFileResponse(String url, Map<String, String> headers, RequestBody fileBody, String paramName, String fileName, Long timeout, TimeUnit timeUnit) throws IOException {
        MultipartBody body = new MultipartBody.Builder()
                .setType(FORM_TYPE)
                .addFormDataPart(paramName, fileName, fileBody)
                .build();

        Request.Builder builder = new Request.Builder();
        if (headers != null) {
            for (Map.Entry<String, String> header : headers.entrySet()) {
                builder.addHeader(header.getKey(), header.getValue());
            }
        }
        Request request = builder
                .post(body)
                .url(url)
                .build();

        return excute(request, timeout, timeUnit);
    }

    private Response excute(Request request, Long timeout, TimeUnit timeUnit) throws IOException {
        OkHttpClient client = getClient(timeout, timeUnit);
        if (isSync()) {
            return client.newCall(request).execute();
        }

        String traceId = HttpUtil.getTraceId(request);
        client.newCall(request).enqueue(new Callback() {
            @Override
            public void onFailure(@NotNull Call call, @NotNull IOException e) {
                log.error(StrUtil.format("异步-traceId-{}失败", traceId), e);
            }

            @Override
            public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
            }
        });
        return null;
    }

    private Response getResponse(String url, Map<String, String> header, String xml, Long timeout, TimeUnit timeUnit, MediaType xmlType) throws IOException {
        RequestBody body = RequestBody.create(xml, xmlType);
        Request.Builder builder = new Request.Builder()
                .url(url)
                .post(body);
        if (header != null) {
            for (Map.Entry<String, String> entry : header.entrySet()) {
                builder.addHeader(entry.getKey(), entry.getValue());
            }
        }

        Request request = builder.build();
        return excute(request, timeout, timeUnit);
    }
}
