package com.starbuds.server.service.net.okhttp;

import cn.hutool.core.util.StrUtil;
import com.starbuds.server.service.net.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import okhttp3.Interceptor;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.ResponseBody;
import okio.Buffer;
import okio.BufferedSource;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 请求日志拦截器
 *
 * @author aoe
 * @date 2020/6/11
 */
@Slf4j
@Component
public class LoggingInterceptor implements Interceptor {

    private ExecutorService executor;

    @Value("${xy.http.LoggingInterceptor.queueCapacity}")
    private int queueCapacity;

    @Value("${xy.http.LoggingInterceptor.executor.corePoolSize}")
    private int corePoolSize;

    @Value("${xy.http.LoggingInterceptor.executor.maximumPoolSize}")
    private int maximumPoolSize;

    @Value("${xy.http.LoggingInterceptor.executor.keepAliveTime}")
    private int keepAliveTime;

    @PostConstruct
    private void init() {
        BlockingQueue queue = new ArrayBlockingQueue(queueCapacity);
        executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, keepAliveTime, TimeUnit.SECONDS,
                queue, new DefaultThreadFactory());
        log.info("OKHttp LoggingInterceptor 配置 queueCapacity={} ExecutorService corePoolSize={} maximumPoolSize={} keepAliveTime={} 秒",
                queueCapacity, corePoolSize, maximumPoolSize, keepAliveTime);
    }

    @Override
    public Response intercept(Interceptor.Chain chain) throws IOException {
        Request request = chain.request();
        String traceId = HttpUtil.getTraceId(request);
        boolean isSync = HttpUtil.getTraceIsSync(request);
        // 发送前清除追踪信息
        request = HttpUtil.cleanTraceInfo(request);

        long t1 = System.currentTimeMillis();
        String reqBody = request.body() == null ? "" : request.body().toString();
        log.info("发送 {} traceId={}  method={} {} headers={} body={}", isSync ? "同步请求" : "异步请求", traceId,
                request.method(), request.url(), request.headers(), reqBody);

        final Response response = chain.proceed(request);

        long t2 = System.currentTimeMillis();
        final long cost = t2 - t1;

        executor.execute(() -> {
            String resBodyStr = null;

            try {
                ResponseBody resBody = response.body();
                if (reqBody == null) {
                    resBodyStr = "";
                } else {
                    BufferedSource source = resBody.source();
                    // request the entire body.
                    source.request(resBody.source().readLong());
                    Buffer buffer = source.getBuffer();
                    // clone buffer before reading from it
                    resBodyStr = buffer.clone().readString(Charset.forName("UTF-8"));
                }
            } catch (IOException e) {
                String errMsg = StrUtil.format("接收 request 失败 traceId={} 耗时={}ms status={} headers={}", traceId, cost,
                        response.code(), response.headers());
                log.error(errMsg, e);
            }
            log.info("接收 request traceId={} 耗时={}ms status={} headers={} body={}", traceId, cost,
                    response.code(), response.headers(), resBodyStr);
        });

        return response;
    }

    /**
     * The default thread factory
     */
    private static class DefaultThreadFactory implements ThreadFactory {
        private static final AtomicInteger poolNumber = new AtomicInteger(1);
        private final ThreadGroup group;
        private final AtomicInteger threadNumber = new AtomicInteger(1);
        private final String namePrefix;

        DefaultThreadFactory() {
            SecurityManager s = System.getSecurityManager();
            group = (s != null) ? s.getThreadGroup() :
                    Thread.currentThread().getThreadGroup();
            namePrefix = "pool-OkHttp-LoggingInterceptor" +
                    poolNumber.getAndIncrement() +
                    "-thread-";
        }

        @Override
        public Thread newThread(Runnable r) {
            Thread t = new Thread(group, r,
                    namePrefix + threadNumber.getAndIncrement(),
                    0);
            t.setDaemon(false);
            t.setPriority(Thread.NORM_PRIORITY);
            return t;
        }
    }
}