package com.starbuds.server.service.net.okhttp;

import lombok.extern.slf4j.Slf4j;
import okhttp3.Dispatcher;
import okhttp3.OkHttpClient;
import org.apache.commons.collections4.map.LRUMap;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * OkHttpClient 工厂
 *
 * @author aoe
 * @date 2020/6/9
 */
@Component
@Slf4j
public class OkHttpClientFactory {

    /**
     * 同步请求客户端
     */
    private OkHttpClient client;

    /**
     * 异步请求客户端
     */
    private OkHttpClient asyncClient;

    /**
     * 同步请求-同时执行的最大请求数
     */
    @Value("${xy.http.sync.maxRequests}")
    private int syncMaxRequests;

    /**
     * 同步请求-超时时间（毫秒）
     */
    @Value("${xy.http.sync.timeout}")
    private long syncTimout;

    /**
     * 异步请求-同时执行的最大请求数
     */
    @Value("${xy.http.async.maxRequests}")
    private int asyncMaxRequests;

    /**
     * 异步请求-超时时间（毫秒）
     */
    @Value("${xy.http.async.timeout}")
    private long asyncTimout;

    private LRUMap<String, OkHttpClient> clientMap;

    @Resource
    private LoggingInterceptor loggingInterceptor;

    @Value("${xy.http.client.queueCapacity}")
    private int queueCapacity;

    @Value("${xy.http.client.executor.corePoolSize}")
    private int corePoolSize;

    @Value("${xy.http.client.executor.maximumPoolSize}")
    private int maximumPoolSize;

    @Value("${xy.http.client.executor.keepAliveTime}")
    private int keepAliveTime;

    @PostConstruct
    private void init() {
        BlockingQueue queue = new ArrayBlockingQueue(queueCapacity);
        ExecutorService executor = new ThreadPoolExecutor(corePoolSize, maximumPoolSize, keepAliveTime, TimeUnit.SECONDS,
                queue, new DefaultThreadFactory());
        log.info("OKHttp Client 配置 queueCapacity={} ExecutorService corePoolSize={} maximumPoolSize={} keepAliveTime={} 秒",
                queueCapacity, corePoolSize, maximumPoolSize, keepAliveTime);
        // 同步请求
        Dispatcher dispatcher = new Dispatcher(executor);
        int maxRequests = syncMaxRequests + asyncMaxRequests;
        dispatcher.setMaxRequests(maxRequests);
        client = new OkHttpClient.Builder()
                .addInterceptor(loggingInterceptor)
                .callTimeout(syncTimout, TimeUnit.MILLISECONDS)
                .dispatcher(dispatcher)
                .build();

        // 异步请求
        Dispatcher asyncDispatcher = new Dispatcher(executor);
        asyncDispatcher.setMaxRequests(asyncMaxRequests);
        asyncClient = client.newBuilder()
                .callTimeout(asyncTimout, TimeUnit.MILLISECONDS)
                .dispatcher(asyncDispatcher)
                .build();

        log.info("总体请求最大执行数量 {}", dispatcher.getMaxRequests());
        log.info("同步请求 最大执行数量 {} 超时时间 {} 毫秒", dispatcher.getMaxRequests() - asyncDispatcher.getMaxRequests(),
                syncTimout);
        log.info("异步请求 最大执行数量 {} 超时时间 {} 毫秒", asyncDispatcher.getMaxRequests(), asyncTimout);

        clientMap = new LRUMap(100);
    }

    /**
     * 创建 OkHttpClient （同步请求）
     *
     * @param timeout  超时时间
     * @param timeUnit 时间单位
     * @return 自定义 client
     */
    public OkHttpClient buildSync(Long timeout, TimeUnit timeUnit) {
        if (timeout == null || timeUnit == null) {
            return client;
        }

        String key = timeout + timeUnit.toString();
        OkHttpClient syncClient = clientMap.get(key);
        if (syncClient != null) {
            return syncClient;
        }

        syncClient = client.newBuilder().callTimeout(timeout, timeUnit).build();
        clientMap.put(key, syncClient);
        return syncClient;
    }

    /**
     * 创建 OkHttpClient （异步步请求）
     *
     * @param timeout  超时时间
     * @param timeUnit 时间单位
     * @return 自定义 client
     */
    public OkHttpClient buildAsync(Long timeout, TimeUnit timeUnit) {
        if (timeout == null || timeUnit == null) {
            return asyncClient;
        }

        String key = "async" + timeout + timeUnit.toString();
        OkHttpClient syncClient = clientMap.get(key);
        if (syncClient != null) {
            return syncClient;
        }

        syncClient = client.newBuilder().callTimeout(timeout, timeUnit).build();
        clientMap.put(key, syncClient);
        return syncClient;
    }

    /**
     * The default thread factory
     */
    private static class DefaultThreadFactory implements ThreadFactory {
        private static final AtomicInteger poolNumber = new AtomicInteger(1);
        private final ThreadGroup group;
        private final AtomicInteger threadNumber = new AtomicInteger(1);
        private final String namePrefix;

        DefaultThreadFactory() {
            SecurityManager s = System.getSecurityManager();
            group = (s != null) ? s.getThreadGroup() :
                    Thread.currentThread().getThreadGroup();
            namePrefix = "pool-OkHttp-Dispatcher" +
                    poolNumber.getAndIncrement() +
                    "-thread-";
        }

        @Override
        public Thread newThread(Runnable r) {
            Thread t = new Thread(group, r,
                    namePrefix + threadNumber.getAndIncrement(),
                    0);
            t.setDaemon(false);
            t.setPriority(Thread.NORM_PRIORITY);
            return t;
        }
    }
}
